@@ -37,8 +37,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include "xio.h"
 
-#if defined(SYS_UNIX) || defined(COMP_MSC) || defined(__APPLE__)
+#if defined(SYS_UNIX) || defined(COMP_MSC) || defined(SYS_APPLE)
 #define lseek64 lseek
 #endif
 
@@ -55,9 +56,12 @@
                    ((x>>8)  & 0x0000ff00) |\
                    ((x<<8)  & 0x00ff0000) |\
                    ((x<<24) & 0xff000000))
+# define SWAP8(x) (( (SWAP4(x)<<32) & 0xffffffff00000000ULL) |\
+                   ( SWAP4(x))
 #else
 # define SWAP2(a) (a)
 # define SWAP4(a) (a)
+# define SWAP8(a) (a)
 #endif
 
 typedef unsigned long DWORD;
@@ -92,11 +96,19 @@ typedef unsigned char boolean;
 #define idx1tag MAKEFOURCC('i','d','x','1')
 #define Tag00db MAKEFOURCC('0','0','d','b')
 #define Tag00dc MAKEFOURCC('0','0','d','c')
+#define Tag01db MAKEFOURCC('0','1','d','b')
+#define Tag01dc MAKEFOURCC('0','1','d','c')
+#define Tag00wb MAKEFOURCC('0','0','w','b')
 #define Tag01wb MAKEFOURCC('0','1','w','b')
 #define Tagwave MAKEFOURCC('f','m','t',' ')
 #define Tagdata MAKEFOURCC('d','a','t','a')
 #define TagDATA MAKEFOURCC('D','A','T','A')
+#define indxtag MAKEFOURCC('i','n','d','x')
 #define Tag00__ MAKEFOURCC('0','0','_','_')
+#define Tagix00 MAKEFOURCC('i','x','0','0')
+#define Tagix01 MAKEFOURCC('i','x','0','1')
+#define Tagix02 MAKEFOURCC('i','x','0','2')
+#define Tagix03 MAKEFOURCC('i','x','0','3')
 
 /* Build a string from a FOURCC number
    (s must have room for at least 5 chars) */
@@ -126,6 +138,8 @@ static DWORD fcc_type;
 #define CCODE  4
 #define HEX16  5
 #define _TAG   6
+#define INT8   7
+#define INT64  8
 
 struct FLAGLIST {
     int  bit;
@@ -223,6 +237,28 @@ struct VAL names_idx1[] = {
     { INT32,  "length" },
     { EoLST,  NULL }    
 };
+struct VAL names_indx[] = {
+    { INT16, "longs/entry"},
+    { INT8, "index_sub_t" },
+    { INT8, "index_t" },
+    { INT32, "entries_used"},
+    { CCODE, "fcc_handler"},
+    { INT32, "reserved1"},
+    { INT32, "reserved2"},
+    { INT32, "reserved3"},
+    { EoLST,  NULL }    
+};
+struct VAL names_stdidx[] = {
+    { INT16, "longs/entry"},
+    { INT8, "index_sub_t" },
+    { INT8, "index_t" },
+    { INT32, "entries_used"},
+    { CCODE, "fcc_handler"},
+    { INT64, "base_offset"},
+    { INT32, "reserved3"},
+    { EoLST,  NULL }    
+};
+
 
 #define MAX_BUF (4096)
 static char buffer[MAX_BUF];
@@ -233,7 +269,7 @@ static size_t avi_read(int fd, char *buf, size_t len)
    size_t r = 0;
 
    while (r < len) {
-      n = read (fd, buf + r, len - r);
+      n = xio_read (fd, buf + r, len - r);
 
       if (n <= 0)
 	  return r;
@@ -249,7 +285,7 @@ static size_t avi_write (int fd, char *buf, size_t len)
    size_t r = 0;
 
    while (r < len) {
-      n = write (fd, buf + r, len - r);
+      n = xio_write (fd, buf + r, len - r);
       if (n < 0)
          return n;
       
@@ -288,16 +324,27 @@ static void dump_vals(int fd, int count, struct VAL *names)
 {
     DWORD i,j,val32;
     WORD  val16;
+    off_t val64;
+    char  val8;
     
     for (i = 0; names[i].type != EoLST; i++) {
 	switch (names[i].type) {
+	case INT8:
+	    xio_read(fd, &val8, 1);
+	    printf("\t%-12s = %d\n",names[i].name,val8);
+	    break;
+	case INT64:
+	    xio_read(fd, &val64, 8);
+	    val64 = SWAP8(val64);
+	    printf("\t%-12s = 0x%016llx\n",names[i].name,val64);
+	    break;
 	case INT32:
-	    read(fd, &val32, 4);
+	    xio_read(fd, &val32, 4);
 	    val32 = SWAP4(val32);
 	    printf("\t%-12s = %ld\n",names[i].name,val32);
 	    break;
 	case CCODE:
-	    read(fd, &val32,4);
+	    xio_read(fd, &val32,4);
 	    val32 = SWAP4(val32);
 	    if (val32) {
 		printf("\t%-12s = %c%c%c%c (0x%lx)\n",names[i].name,
@@ -311,7 +358,7 @@ static void dump_vals(int fd, int count, struct VAL *names)
 	    }
 	    break;
 	case _TAG:
-	    read(fd, &val32,4);
+	    xio_read(fd, &val32,4);
 	    val32 = SWAP4(val32);
 	    if (val32) {
 		printf("\t%-12s = %c%c%c%c\n",names[i].name,
@@ -324,7 +371,7 @@ static void dump_vals(int fd, int count, struct VAL *names)
 	    }
 	    break;
 	case FLAGS:
-	    read(fd, &val32,4);
+	    xio_read(fd, &val32,4);
 	    val32 = SWAP4(val32);
 	    printf("\t%-12s = 0x%lx\n",names[i].name,val32);
 	    if (names[i].flags) {
@@ -335,13 +382,13 @@ static void dump_vals(int fd, int count, struct VAL *names)
 	    }
 	    break;
 	case INT16:
-	    read(fd, &val16,2);
+	    xio_read(fd, &val16,2);
 	    val16 = SWAP2(val16);
 	    printf("\t%-12s = %ld\n",names[i].name,(long)val16);
 	    break;
 
 	case HEX16:
-	    read(fd, &val16,2);
+	    xio_read(fd, &val16,2);
 	    val16 = SWAP2(val16);
 	    printf("\t%-12s = 0x%lx\n",names[i].name,(long)val16);
 	    break;
@@ -349,6 +396,7 @@ static void dump_vals(int fd, int count, struct VAL *names)
     }
 }
 
+#if 0
 static void hexlog(unsigned char *buf, int count)
 {
     int l,i;
@@ -370,7 +418,7 @@ static void hexlog(unsigned char *buf, int count)
 	printf("\n");
     }
 }
-
+#endif
 static unsigned char*
 off_t_to_char(off_t val, int base, int len)
 {
@@ -392,8 +440,8 @@ off_t_to_char(off_t val, int base, int len)
 
 static boolean ReadChunkHead(int fd, FOURCC* ID, DWORD* size)
 {
-    if (!read(fd, ID, sizeof(FOURCC))) return(FALSE);
-    if (!read(fd, size, sizeof(DWORD))) return(FALSE);
+    if (!xio_read(fd, ID, sizeof(FOURCC))) return(FALSE);
+    if (!xio_read(fd, size, sizeof(DWORD))) return(FALSE);
 
     *size = SWAP4(*size);
 
@@ -432,7 +480,7 @@ static boolean ProcessChunk(int fd, off_t filepos, off_t filesize,
 	printf("  *****  Error: Data would be behind end of file!\n");
     }
     
-    tt=lseek64(fd, filepos, SEEK_SET); /* Go to desired file position!     */
+    tt=xio_lseek(fd, filepos, SEEK_SET); /* Go to desired file position!     */
 
 #ifdef AVI_DEBUG
     printf("lseek64=%Lu/%Lu (%Lu)\n", tt, filepos, filesize); 
@@ -510,7 +558,7 @@ static boolean ProcessChunk(int fd, off_t filepos, off_t filesize,
 #ifdef AVI_DEBUG
 	    printf("enter [%d] size=%lu pos=%Lu left=%lu\n", RekDepth, subchunksize, datapos, *chunksize-datashowed); 
 #endif
-	    if (!ProcessChunk(fd, datapos, filesize, (off_t) 0,
+	    if (!ProcessChunk(fd, datapos, filesize,  0,
 			      RekDepth+1, &subchunksize)) return(FALSE);
 	    
 	    subchunklen = sizeof(FOURCC) +  /* this is the complete..   */
@@ -534,7 +582,7 @@ static boolean ProcessChunk(int fd, off_t filepos, off_t filesize,
     case strhtag:
     {
 	char   typestr[5];
-	read(fd, &fcc_type,sizeof(FOURCC));
+	xio_read(fd, &fcc_type,sizeof(FOURCC));
 	FOURCC2Str(fcc_type,typestr);
 	printf("\tfcc_type     = %s\n",typestr);
 	dump_vals(fd,sizeof(names_strh)/sizeof(struct VAL),names_strh);
@@ -559,15 +607,64 @@ static boolean ProcessChunk(int fd, off_t filepos, off_t filesize,
 	dump_vals(fd,sizeof(names_strf_auds)/sizeof(char*),names_strf_auds);
 	break;
 
-    case idx1tag:
+    case indxtag: {
+	long chunks=*chunksize-sizeof(names_indx)/sizeof(char*);
+	off_t offset;
+	DWORD size, duration;
+	long u=0;
+	off_t indxend = datapos + chunks;
+	dump_vals(fd,sizeof(names_indx)/sizeof(char*),names_indx);
 
-	while (datapos<filesize) {
+	while (datapos < indxend) {
+	    xio_read(fd, &offset,8);
+	    xio_read(fd, &size, 4);
+	    xio_read(fd, &duration,4);
+	    offset = SWAP8(offset);
+	    size = SWAP4(size);
+	    duration = SWAP4(duration);
+	    if (size!=0)
+	    printf("\t\t [%6ld] 0x%016llx 0x%08lx %8d\n", u++, offset, size,
+		    duration);
+	    datapos += 16;
+	}
+	break;
+	}
+    case Tagix00:
+    case Tagix01:
+    case Tagix02:
+    case Tagix03: {
+	long chunks=*chunksize-sizeof(names_stdidx)/sizeof(char*);
+	unsigned int offset, size, key;
+	long u=0;
+	off_t indxend = datapos + chunks;
+	dump_vals(fd,sizeof(names_stdidx)/sizeof(char*),names_stdidx);
+
+	while (datapos < indxend) {
+	    xio_read(fd, &offset,4);
+	    xio_read(fd, &size, 4);
+	    offset = SWAP4(offset);
+	    size = SWAP4(size);
+	    key  = size;
+	    key  = key&0x80000000?0:1;
+	    size &= 0x7fffffff;
+	    if (size!=0)
+	    printf("\t\t [%6ld] 0x%08lx 0x%08lx key=%s\n", u++, offset, size,
+		    key?"yes":"no");
+	    datapos += 8;
+	}
+		  }
+	break;
+
+    case idx1tag: {
+	off_t idxend = datapos+*chunksize;
+
+	while (datapos<idxend) {
 
 	    DWORD val32;
 	    static long u=0;
 
 	    //tag:
-	    read(fd, &val32,4);
+	    xio_read(fd, &val32,4);
 	    val32 = SWAP4(val32);
 	    if(val32==Tag00db) {
 		printf("\t\t [%6ld] %s=%c%c%c%c ", u++, "tag",
@@ -583,22 +680,23 @@ static boolean ProcessChunk(int fd, off_t filepos, off_t filesize,
 	    
 	    
 	    //flag
-	    read(fd, &val32, 4);
+	    xio_read(fd, &val32, 4);
 	    val32 = SWAP4(val32);
 	    printf("flags=%02ld ",val32);
 
 	    //pos
-	    read(fd, &val32, 4);
+	    xio_read(fd, &val32, 4);
 	    val32 = SWAP4(val32);
 	    printf("0x%08lx",val32);
 
 	    //size
-	    read(fd, &val32, 4);
+	    xio_read(fd, &val32, 4);
 	    val32 = SWAP4(val32);
 	    printf("%8ld\n",val32);
 
 	    datapos+=16;
 	}
+	}
 	break;
 
     case dmlhtag:
@@ -623,7 +721,7 @@ static boolean DumpChunk(int fd, off_t filepos, off_t filesize,
     if (filepos>filesize-1)  /* Oops. Must be something wrong!      */
 	return(FALSE);
     
-    lseek64(fd, filepos, SEEK_SET);
+    xio_lseek(fd, filepos, SEEK_SET);
 
     if (!ReadChunkHead(fd, &chunkid,chunksize)) {  /* read chunk header */
       return(FALSE);
@@ -664,7 +762,7 @@ static boolean DumpChunk(int fd, off_t filepos, off_t filesize,
       char   formstr[5];     /* format of chunk converted to string */
       DWORD  subchunksize;   /* size of a read subchunk             */
       
-      read(fd, &formtype,sizeof(FOURCC));    /* read the form type */
+      xio_read(fd, &formtype,sizeof(FOURCC));    /* read the form type */
       FOURCC2Str(formtype,formstr);           /* make it printable  */
       
       datashowed=sizeof(FOURCC);    /* we showed the form type      */
@@ -690,9 +788,10 @@ static boolean DumpChunk(int fd, off_t filepos, off_t filesize,
     break;
     
     case Tag01wb:
+    case Tag00wb:
 
       if(mode==1) {
-	lseek(fd, datapos, SEEK_SET);
+	xio_lseek(fd, datapos, SEEK_SET);
 	if(avi_read_write(fd, STDOUT_FILENO, size)!=size) return(FALSE);
       }
       if(eos) return(FALSE);
@@ -701,10 +800,12 @@ static boolean DumpChunk(int fd, off_t filepos, off_t filesize,
       
     case Tag00db:
     case Tag00dc:
+    case Tag01db:
+    case Tag01dc:
     case Tag00__:
 
       if(mode==0) {
-	lseek(fd, datapos, SEEK_SET);
+	xio_lseek(fd, datapos, SEEK_SET);
 	if(avi_read_write(fd, STDOUT_FILENO, size)!=size) return(FALSE);
       }
     
@@ -716,7 +817,7 @@ static boolean DumpChunk(int fd, off_t filepos, off_t filesize,
     case TagDATA:
       
       if(mode==2) {
-	lseek(fd, datapos, SEEK_SET);
+	xio_lseek(fd, datapos, SEEK_SET);
 	if(avi_read_write(fd, STDOUT_FILENO, size)!=size) return(FALSE);
       }
       
@@ -741,15 +842,15 @@ int AVI_scan(char *file_name)
 #if defined(SYS_WINDOWS)
     if (!(fd=open(file_name, O_RDONLY|O_BINARY))) {
 #else
-    if (!(fd=open(file_name, O_RDONLY))) {
+    if (!(fd=xio_open(file_name, O_RDONLY))) {
 #endif
 	printf("\n\n *** Error opening file %s. Program aborted!\n",
 	       file_name);
 	return(1);
     }
 
-    filesize = lseek64(fd, 0, SEEK_END);
-    lseek64(fd, 0, SEEK_SET);
+    filesize = xio_lseek(fd, 0, SEEK_END);
+    xio_lseek(fd, 0, SEEK_SET);
 
     printf("Contents of file %s (%s/", file_name,
 	   off_t_to_char(filesize,10,1));
@@ -763,7 +864,7 @@ int AVI_scan(char *file_name)
 	printf("\n");
     }
 
-    close(fd);
+    xio_close(fd);
 
     return(0);
 }
@@ -780,11 +881,11 @@ int AVI_dump(char *file_name, int mode)
 #if defined(SYS_WINDOWS)
     if (!(fd=open(file_name,O_RDONLY|O_BINARY))) return(1);
 #else
-    if (!(fd=open(file_name,O_RDONLY))) return(1);
+    if (!(fd=xio_open(file_name,O_RDONLY))) return(1);
 #endif
 
-    filesize = lseek64(fd, 0, SEEK_END);
-    lseek64(fd, 0, SEEK_SET);
+    filesize = xio_lseek(fd, 0, SEEK_END);
+    xio_lseek(fd, 0, SEEK_SET);
 
     for (filepos = 0; filepos < filesize;) {
 	chunksize = 0;
@@ -793,7 +894,7 @@ int AVI_dump(char *file_name, int mode)
 	filepos += chunksize + 8;
     }
 
-    close(fd);
+    xio_close(fd);
 
     return(0);
 }
