@@ -150,6 +150,8 @@ static void usage(void)
     "  --omit-empty-packets     Omit the empty subtitle packets.\n"
     "  --old-headers            Assume OGM files have been created with\n"
     "                           an older version of ogmmerge ( < 1.1).\n"
+    "  --nav-seek <filename>    Use an external AVI index file as generated by\n"
+    "                           aviindex from transcode.\n"
     "\n"
     " Other options:\n"
     "  -l, --list-types         Lists supported input file types.\n"
@@ -500,7 +502,7 @@ static void parse_args(int argc, char **argv)
   filelist_t      *file;
   audio_sync_t     async;
   range_t          range;
-  char           **comments, *fourcc;
+  char           **comments, *fourcc, *nav_seek;
   vorbis_comment  *chapters;
 
   noaudio = 0;
@@ -514,6 +516,7 @@ static void parse_args(int argc, char **argv)
   async.linear = 1.0;
   comments = NULL;
   fourcc = NULL;
+  nav_seek = NULL;
   chapters = NULL;
   for (i = 1; i < argc; i++)
     if (!strcmp(argv[i], "-V") || !strcmp(argv[i], "--version")) {
@@ -629,6 +632,13 @@ static void parse_args(int argc, char **argv)
       omit_empty_packets = true;
     else if (!strcmp(argv[i], "--old-headers")) {
       old_headers = 1;
+    } else if (!strcmp(argv[i], "--nav-seek")) {
+      if ((i + 1) >= argc) {
+        fprintf(stderr, "Error: --nav-seek lacks the file name.\n");
+        exit(1);
+      }
+      nav_seek = argv[i+1];
+      i++;
     }
 #ifdef ENABLE_INDEX
     else if (!strcmp(argv[i], "-i") || !strcmp(argv[i], "--index"))
@@ -692,7 +702,8 @@ static void parse_args(int argc, char **argv)
             if (tstreams != NULL)
               fprintf(stderr, "Warning: -t/-T are ignored for AVI files.\n");
             file->reader = new avi_reader_c(file->name, astreams, vstreams,
-                                            &async, &range, comments, fourcc);
+                                            &async, &range, comments, fourcc,
+                                            nav_seek);
             break;
           case TYPEWAV:
             if ((astreams != NULL) || (vstreams != NULL) ||
@@ -784,6 +795,10 @@ static void parse_args(int argc, char **argv)
         free(vstreams);
         vstreams = NULL;
       }
+      if (tstreams != NULL) {
+        free(tstreams);
+        tstreams = NULL;
+      }
       async.displacement = 0;
       async.linear = 1.0;
       memset(&range, 0, sizeof(range_t));
