@@ -54,8 +54,8 @@ int avi_reader_c::probe_file(FILE *file, off_t size) {
  */
 avi_reader_c::avi_reader_c(char *fname, unsigned char *astreams,
                            unsigned char *vstreams, audio_sync_t *nasync,
-                           range_t *nrange, char **ncomments, char *nfourcc)
-                           throw (error_c) {
+                           range_t *nrange, char **ncomments, char *nfourcc,
+                           char* nav_seek) throw (error_c) {
   int            fsize, i;
   off_t          size;
   FILE          *f;
@@ -80,8 +80,11 @@ avi_reader_c::avi_reader_c(char *fname, unsigned char *astreams,
   if (verbose)
     fprintf(stderr, "Using AVI demultiplexer for %s. Opening file. This "
             "may take some time depending on the file's size.\n", fname);
-  rederive_keyframes = 0;
-  if ((avi = AVI_open_input_file(fname, 1)) == NULL) {
+  if (nav_seek == NULL)
+    avi = AVI_open_input_file(fname, 1);
+  else
+    avi = AVI_open_input_indexfile(fname, 0, nav_seek);
+  if (avi == NULL) {
     const char *msg = "avi_reader: Could not initialize AVI source. Reason: ";
     char *s, *error;
     error = AVI_strerror();
@@ -101,6 +104,8 @@ avi_reader_c::avi_reader_c(char *fname, unsigned char *astreams,
   else
     this->vstreams = NULL;
 
+  this->nav_seek = nav_seek;
+
   if (ncomments == NULL)
     comments = ncomments;
   else
@@ -297,25 +302,26 @@ int avi_reader_c::add_audio_demuxer(avi_t *avi, int aid) {
 int avi_reader_c::is_keyframe(unsigned char *data, long size, int suggestion) {
   int i;
 
-  if (!rederive_keyframes)
-    return suggestion;
+//   if (!rederive_keyframes)
+//     return suggestion;
 
   switch (is_divx) {
-    case RAVI_DIVX3:
-      i = *((int *)data);
-      return ((i & 0x40000000) ? 0 : 1);
+//     case RAVI_DIVX3:
+//       i = *((int *)data);
+//       return ((i & 0x40000000) ? 0 : 1);
     case RAVI_MPEG4:
-      for (i = 0; i < size - 5; i++) {
-        if ((data[i] == 0x00) && (data[i + 1] == 0x00) && (data[i + 2] == 0x01) &&
-            (data[i + 3] == 0xb6)) {
-          if ((data[i + 4] & 0xc0) == 0x00)
+      for(i=0; i < size - 5; i++) {
+        if ((data[i] == 0x00) && (data[i + 1] == 0x00) && (data[i + 2] == 0x01)) {
+          if (data[i + 3] == 0x00)
             return 1;
-          else
+           
+          if ((data[i + 3] == 0xb6) && (data[i+4] & 0xc0) == 0)
+            return 1;
+           
             return 0;
         }
-      }
-
       return 0;
+    }
     default:
       return suggestion;
   }
@@ -571,4 +577,3 @@ void avi_reader_c::display_progress() {
   }
   fflush(stdout);
 }
-
